<?php
// Changes start here ---------------------------------------------------------
/**
 * Returns the DPI in the specified JPEG image header.
 * @param $path Path of the JPEG image
 * @return DPI of the JPEG image (OR zero)
 */
function imageGetDPI($path) {
    $result = 0;

    // if Path was specified ...
    if(strlen($path)) {

        // ... load enough of the JPEG Header to determine DPI
        $file = @fopen($path, "r");  
        $header = @fread($file, 18);  
        @fclose($file);

        // if image is JPEG/JFIF (APP0) format ...
        if(ord($header[2]) == 0xff && ord($header[3]) == 0xe0) {

            // ... if resolution is DPI ...
            if(ord($header[13]) == 1) { 

                // ... extract the DPIs
                $xdpi = (ord($header[14]) * 256);
                $xdpi += ord($header[15]);
                $ydpi = (ord($header[16]) * 256);
                $ydpi += ord($header[17]);

                // if X and Y DPI are different ...
                if($xdpi != $ydpi)
                    
                    // ... return both DPIs (as array of integer)
                    $result = array($xdpi, $ydpi);

                // ... otherwise, return common DPI (as integer)
                else
                    $result = $xdpi;
            }
        }

        // ... otherwise, if image is JPEG/EXIF (APP1) format ...
        else if(ord($header[2]) == 0xff && ord($header[3]) == 0xe1) { 

            // ... read EXIF data
			if(function_exists(exif_read_data)) { 
				$exif = exif_read_data($path, "IFD0");

				// if EXIF data exists using DPI ...
				if($exif !== false && $exif['ResolutionUnit'] == 2) {

					// ... extract the DPIs
					list($xdpi) = explode('/', $exif['XResolution']);
					list($ydpi) = explode('/', $exif['YResolution']);

					// if X and Y DPI are different ...
					if($xdpi != $ydpi)

						// ... return both DPIs (as array of integer)
						$result = array((int) $xdpi, (int) $ydpi);

					// ... otherwise, return common DPI (as integer)
					else
						$result = (int) $xdpi;
				}
			}
		}
    }

    // return the DPI (OR zero)
    return $result;
}

/**
 * Sets the DPI in the specified JPEG image header.
 * @param $path Path of the JPEG image
 * @param $dpi DPI value(s) to be set
 */
function imageSetDPI($path, $dpi) {

    // if Path and DPI were specified ...
    if(strlen($path) && $dpi !== null) {

        // ... if DPI is an array ...
        if(is_array($dpi)) {

            // ... get X (and possibly Y) DPI
            $xdpi = $ydpi = $dpi[0];
            if(count($dpi) > 1)
                $ydpi = $dpi[1];
        }

        // ... otherwise, use common DPI
        else
            $xdpi = $ydpi = $dpi;

        // if X and Y DPI are numeric ...
        if(is_int($xdpi) && is_int($ydpi)) {
			
            // ... load the JPEG Image
            $size = @filesize($path);
            $image = @file_get_contents($path);

            // if Image data loaded ...
            if(strlen($image) >= 18) {

                // ... if image is JPEG/JFIF (APP0) format ...
                if(ord($image[2]) == 0xff && ord($image[3]) == 0xe0) { 

                    // ... update DPI information in the JPEG header
                    $image[13] = chr(1);
                    $image[14] = chr(floor($xdpi/256));
                    $image[15] = chr($xdpi%256);
                    $image[16] = chr(floor($ydpi/256));
                    $image[17] = chr($ydpi%256);
  				
                    // re-write the Image
                    $file = @fopen($path, "w");
                    @fwrite($file, $image, $size);
                    @fclose($file);
                }
            }
        }
    }
}
// Changes end here -----------------------------------------------------------

include "setup.php";
session_start();
error_reporting(E_ALL ^ E_NOTICE);
header("Cache-control: private"); 
include "pc_inc/functions.php"; 
include "pc_inc/pc_functions.php";
$dbcon = dbConnect($setup);
$isp = explode("|", $_REQUEST['image']);
anString($isp[0]);
$pic = doSQL("pics", "*, date_format(DATE_ADD(pic_date, INTERVAL 0 HOUR), '%M %e, %Y ')  AS pic_date", " WHERE pic_id='".$isp[0]."' ");
 if(empty($pic['pic_id'])) { die();}
$gallery = doSQL("galleries", "*", " WHERE gal_id='".$pic['pic_gal']."' ");
$returnval = galleryTree($gallery,"pagetitle");
$main_gallery = $returnval[2];
if(empty($main_gallery)) { 
	$main_gal_id = $gallery['gal_id'];
	$gal_date = $gallery['gal_date'];
	$gal_free_image = $gallery['gal_free_image'];
	$gal_client = $gallery['gal_client'];
$main_gallery = $gallery['gal_id'];
	$remove_watermark = $gallery['gal_remove_watermark'];
} else {
	$main_gal = doSQL("galleries", "*, date_format(DATE_ADD(gal_expire, INTERVAL 2 HOUR), '%M %e, %Y ')  AS gal_show_expire", "WHERE gal_id='".$main_gallery."' ");
	$main_gal_id = $main_gal['gal_id'];
	$gal_date = $main_gal['gal_date'];
	$gal_free_image = $main_gal['gal_free_image'];
	$gal_client = $main_gal['gal_client'];
	if($gallery['gal_remove_watermark'] == "1") {
		$remove_watermark = $gallery['gal_remove_watermark'];
	} else {
		$remove_watermark = $main_gal['gal_remove_watermark'];
	}
}
if($gal_free_image!=="1") { die("sorry, an error has occured"); }

if((($gal_client == "1") OR($gal_client == "3"))AND($_SESSION['pc_admin_login'] !== "1")==true) {
	if(isLoggedIn()==true) {	
		$user = countIt("pc_users", "WHERE user_gallery='$main_gallery' AND user_client='".$_SESSION['pc_client_id']."' ");
	}
	if(($user <= 0) && (publicRestrictedAccess($main_gallery))!== true){
	//	print_r($_SESSION);
		exit();
	}
}

insertSQL("pc_free_downloads","dl_pic='".$pic['pic_id']."', dl_date=NOW(), dl_main_gal='$main_gal_id', dl_client='".$_SESSION['pc_client_id']."', dl_ip='".$_SERVER['REMOTE_ADDR']."' ");

if($isp[1] == "".MD5("dis")."") {
	$selImage = $pic['pic_pic'];
} elseif($isp[1] == "".MD5("zm")."") {
	$selImage = $pic['pic_zoom'];
} elseif($isp[1] == "".MD5("dl")."") {
	if(!empty($pic['pic_full'])) {
		$selImage = $pic['pic_full'];
	} else {
		$selImage = $pic['pic_zoom'];
	}
} else {
	die("error");
}
if(empty($selImage)) {
	$selImage = $pic['pic_pic'];
}

if($isp[2] == "".MD5("rmvwm")."") {
	$remove_watermark = "1";
} else {
	$remove_watermark = "0";
}
$theImage = "".$setup['gallery_folder']."/".$gallery['gal_folder'].""."/".$selImage."";
$theImagePath = "".$setup['path']."/".$theImage."";
$size= @GetImageSize($theImagePath); 
$bw = $isp[3];
$colorize = $isp[4];
$colorize_type = $isp[5];
$crop = $isp[6];
$width=$isp[7];
$height=$isp[8];

if($crop == 1) {
	$iwidth=$width;
	$iheight=$height;
} else {
	$iwidth=$size[0];
	$iheight=$size[1];
}

// Changes start here ---------------------------------------------------------

// this is never referenced but I'll keep it here, just in case
$im_src = null;

// assume we will NOT need to use GD library
$new = null;

// Changes end here -----------------------------------------------------------

$wm = doSQL("pc_watermarking", "*", "");

 if(!empty($wm['wm_images_file'])) {
	 if($remove_watermark!=="1") {
		$location = $wm['wm_images_location'];
		$overlay = $setup['path']."/".$wm['wm_images_file'];
		$w_offset = 0;
		$h_offset = 0;
		$overlay = imagecreatefrompng($dir . $overlay);
		 
		// Get the size of overlay
		$owidth = imagesx($overlay);
		$oheight = imagesy($overlay);

// Changes start here ---------------------------------------------------------

        // if GD library has NOT been used yet ...
        if($new == null)

            // ... load the original image into GD
            $new = imagecreatefromjpeg($theImagePath);

// Changes end here -----------------------------------------------------------
		 if($location == "tile") { 
			imagesettile($new, $overlay);
			imagefilledrectangle($new, -0, 0, $iwidth, $iheight, IMG_COLOR_TILED);
		 } elseif($location == "center") {
			// center
			imagecopy($new, $overlay, ($iwidth / 2) - (($owidth - $w_offset) / 2), ($iheight / 2)- (($oheight - $h_offset)/2), 0, 0, $owidth, $oheight);
		 } elseif($location == "bright") {
		   // bottom right
			imagecopy($new, $overlay, $iwidth- $owidth - $w_offset, $iheight - $oheight - $h_offset, 0, 0, $owidth, $oheight);
		 } elseif($location == "bottom") {
			// bottom center
			imagecopy($new, $overlay, ($iwidth / 2) - (($owidth - $w_offset) / 2), $iheight - $oheight - $h_offset, 0, 0, $owidth, $oheight);
		 } elseif($location == "bleft") {
			// bottom left
			imagecopy($new, $overlay, 0, $iheight - $oheight - $h_offset, 0, 0, $owidth, $oheight);

		 } elseif($location == "uright") {
		   // top right
			imagecopy($new, $overlay, $iwidth- $owidth - $w_offset, 0, 0, 0, $owidth, $oheight);
		 } elseif($location == "top") {
			// top center
			imagecopy($new, $overlay, ($iwidth / 2) - (($owidth - $w_offset) / 2), 0, 0, 0, $owidth, $oheight);
		 } elseif($location == "uleft") {
			// top left
			imagecopy($new, $overlay, 0, 0, 0, 0, $owidth, $oheight);
		 } else {
			imagecopy($new, $overlay, ($iwidth / 2) - (($owidth - $w_offset) / 2), ($iheight / 2)- (($oheight - $h_offset)/2), 0, 0, $owidth, $oheight);
		 }
		imagedestroy($overlay);
	 }
 }

 if(($wm['wm_add_logo']=="1")AND(!empty($wm['wm_logo_file']))AND($remove_watermark!=="1")==true) {
	 $location = $wm['wm_add_logo_location'];
	$overlay =$setup['path']."/".$wm['wm_logo_file'];
	$w_offset = 0;
	$h_offset = 0;
	$overlay = imagecreatefrompng($dir . $overlay);
	$owidth = imagesx($overlay);
	$oheight = imagesy($overlay);

// Changes start here ---------------------------------------------------------

    // ... if GD library has NOT been used yet ...
    if($new == null)

        // ... load the original image into GD
        $new = imagecreatefromjpeg($theImagePath);

// Changes end here -----------------------------------------------------------

	 if($location == "center") {
		// center
		imagecopy($new, $overlay, ($iwidth / 2) - (($owidth - $w_offset) / 2), ($iheight / 2)- (($oheight - $h_offset)/2), 0, 0, $owidth, $oheight);
	 } elseif($location == "bright") {
	   // bottom right
		imagecopy($new, $overlay, $iwidth- $owidth - $w_offset, $iheight - $oheight - $h_offset, 0, 0, $owidth, $oheight);
	 } elseif($location == "bottom") {
		// bottom center
		imagecopy($new, $overlay, ($iwidth / 2) - (($owidth - $w_offset) / 2), $iheight - $oheight - $h_offset, 0, 0, $owidth, $oheight);
	 } elseif($location == "bleft") {
		// bottom left
		imagecopy($new, $overlay, 0, $iheight - $oheight - $h_offset, 0, 0, $owidth, $oheight);

	 } elseif($location == "uright") {
	   // top right
		imagecopy($new, $overlay, $iwidth- $owidth - $w_offset, 0, 0, 0, $owidth, $oheight);
	 } elseif($location == "top") {
		// top center
		imagecopy($new, $overlay, ($iwidth / 2) - (($owidth - $w_offset) / 2), 0, 0, 0, $owidth, $oheight);
	 } elseif($location == "uleft") {
		// top left
		imagecopy($new, $overlay, 0, 0, 0, 0, $owidth, $oheight);
	 } else {
		imagecopy($new, $overlay, ($iwidth / 2) - (($owidth - $w_offset) / 2), ($iheight / 2)- (($oheight - $h_offset)/2), 0, 0, $owidth, $oheight);
	 }
	imagedestroy($overlay);
 }

$theimage= "".$pic['pic_org']."";
header("Content-transfer-encoding: binary\n"); 
header("Content-Type: image/jpeg"); 
header("Content-Disposition: attachment; filename=\"$theimage\" ");
header('Cache-control: no-cache');

// Changes start here ---------------------------------------------------------

// if GD library has been used ...
if($new != null) {

    // ... write image to temp file (MUST use a file to preserve the DPI!)
    $tmp = tempnam("/tmp", "FREE-");
    imageJPEG($new, $tmp, 95);
    imageDestroy($new);

    // set the original DPI into the copy
    imageSetDPI($tmp, imageGetDPI($theImagePath));

    // return the copy
    @readfile($tmp);

    // discard the temp file
    @unlink($tmp);
}

// ... otherwise, just return the original (uses NO GD so faster, no memory use)
else
    @readfile($theImagePath);

// Changes end here -----------------------------------------------------------

exit();
?>